/***************************************************************
	BILLIARD

	Filip Hammerstad. Spring 2004

	components: 
		-OpenGL
		-GLU
		-GLUT
		-Tokamak Physics library(www.tokamakphysics.com)


   Revised june 2004: Texture usage removed for simplicity
   B.Stenseth

*****************************************************************/

#include <iostream>
#include <cstdlib>
#include <GL/glut.h>
#include "../tokamak/include/tokamak.h"
#include <time.h>

#define FLOORSIZE 40.0f

using namespace std;

bool useLight = true; //light or no light?

//materials
GLfloat mat_shininess[] = {50.0};
GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
GLfloat mat_table[] = {0.0, 0.3, 0.0, 1.0};
GLfloat mat_walls[] = {0.5, 0.3, 0.1, 1.0};
GLfloat mat_ball[] = {1.0, 1.0, 1.0, 1.0};
GLfloat mat_balls[] = {1.0, 0.0, 0.0, 1.0};

GLfloat impx = 0.0;
GLfloat impy = 0.0;
GLfloat impz = 0.0;

int imp [4] = {0,0,0,0};
neV3  impulse;

// Timer variables
bool gbUseHFTimer;
INT64 gCurrentTime;
float gfTimeScale;

//the simulator
neSimulator* gSim = NULL;

neV3 gravity = {0.0,0.0,0.0};	// A vector to store the direction and intensity of gravity

//the physics bodies
neRigidBody* gBall = NULL;
neRigidBody* gBalls[15] = {NULL};
neRigidBody* gStick = NULL;
neAnimatedBody* gFloor = NULL;
neAnimatedBody* gRwall = NULL;
neAnimatedBody* gLwall = NULL;
neAnimatedBody* gBwall = NULL;
neAnimatedBody* gFwall = NULL;

static int win;

//translate and rotate variables

GLfloat globalx = -7.0;
GLfloat globaly = -25.0;
GLfloat globalz = -20.;
GLfloat rotx = 1.0;
GLfloat roty = 1.0;
GLfloat anglx = 32.9;
GLfloat angly = -11.2;

//bool variables for which rotations and translations are active, used in idle and keyboard function
bool rot_up,rot_down,rot_right,rot_left,xup,xdown,yup,ydown,zup,zdown = false;
 
//quadric objects
GLUquadricObj* q;
GLUquadricObj* qBalls[15];


// function prototypes
void disp(void);
void idle(void);
void keyb(unsigned char key, int x, int y);
void keyUp(unsigned char key, int x, int y);
void mouseb(int button, int state, int x, int y);
void reshape(int w, int h);
void drawTable(int texIndex); //draws all 5 surfaces of the table(floor + 4 walls)
void drawBall(neRigidBody* localBall, float radius, GLUquadricObj* localQ, int texIndex); // draws a ball
void drawStick(neRigidBody* localStick, float radius, float length, int texIndex); // draws a Stick
void drawCube(GLfloat w, GLfloat he, GLfloat len); //draws a cube of w width(x), he height(y) and len length(z)


// Initialise a timer for use in our application.
// Wherever possible use a Performance timer as this
// gives the best accuracy.
// If this isn't available, fall back to the multimedia timer.
// In each case, note the number of "ticks" per second so that
// we can divide the elapsed time by this value to determine
// how much real time has actually passed.
// For more information on using these timers, see the articles at
// www.mvps.org/directx
bool InitTimer(void)
{
	INT64 TimerFrequency;

	// Can we use the high performance timer?
	if (QueryPerformanceFrequency( (LARGE_INTEGER*)&TimerFrequency))
	{
		// High performance timer available, let's get ready to use it
		gfTimeScale = 1.0f / TimerFrequency;
		QueryPerformanceCounter((LARGE_INTEGER *) &gCurrentTime);
		gbUseHFTimer = true;
		return true;
	}
	else
	{
		// No high precision timer to be seen, let's use the Windows multimedia timer instead
		gfTimeScale = 0.001f;
		gCurrentTime = timeGetTime();
		gbUseHFTimer = false;
		return true;
	}

	// No timer available at all (should never happen)
	return false;
}


// Use the timer we previously initialised to determine
// how much time (in seconds, or a fraction thereof) has
// passed since this function was last called.
float GetElapsedTime()
{
	INT64 newTime;
	float fElapsed;

	if (gbUseHFTimer) 
	{
		QueryPerformanceCounter((LARGE_INTEGER *) &newTime);
	}
	else 
	{
		newTime=timeGetTime();
	}

	// Scale accordingly
	fElapsed = (float)((newTime - gCurrentTime) * gfTimeScale);

	// Save the new time value for the next time we're called
	gCurrentTime = newTime;

	return fElapsed;
}



// Initialise the Tokamak physics engine.
bool InitPhysics(void)
{
  neGeometry *geom;	// A Geometry object which used to define the shape/size of objects
  neV3 boxSize;		// A variable to store the length, width and height of the boxes ()
  
  neV3 pos;			// The position
  f32 mass;			// The mass 
  neSimulatorSizeInfo sizeInfo;	// Stores data about how many objects we are going to model


  // Create and initialise the simulator
  // Tell the simulator how many rigid bodies we have
  sizeInfo.rigidBodiesCount = 16;
  // Tell the simulator how many animated bodies we have
  sizeInfo.animatedBodiesCount = 5;
  // Tell the simulator how many bodies we have in total
  s32 totalBody = sizeInfo.rigidBodiesCount + sizeInfo.animatedBodiesCount;
  sizeInfo.geometriesCount = totalBody;
  // The overlapped pairs count defines how many bodies it is possible to be in collision
  // at a single time. The SDK states this should be calculated as:
  //   bodies * (bodies-1) / 2
  // So we'll take its word for it. :-)
  sizeInfo.overlappedPairsCount = totalBody * (totalBody - 1) / 2;
  
	// We're not using any of these so set them all to zero
  sizeInfo.rigidParticleCount = 0;
  sizeInfo.constraintsCount = 0;
  sizeInfo.terrainNodesStartCount = 0;

	// Set the gravity. Try changing this to see the effect on the objects
  gravity.Set(0.0f, -10.0f, 0.0f);
	

	// Ready to go, create the simulator object
  gSim = neSimulator::CreateSimulator(sizeInfo, NULL, &gravity);
	
	//set materials in the simulator object
	gSim->SetMaterial(0, 0.4, 0.5); //table material 
	gSim->SetMaterial(1, 0.4, 0.5); //ball material


	/* **** Create rigid body for white ball **** */

  gBall = gSim->CreateRigidBody();

	// Add geometry to the body and set it to be a sphere with a diameter
  geom = gBall->AddGeometry();

  geom->SetSphereDiameter(0.8f);

	geom->SetMaterialIndex(1); //rubber ball material
  // Update the bounding info of the object -- must always call this
  // after changing a body's geometry.
  gBall->UpdateBoundingInfo();

	// Set other properties of the object (mass, position, etc.)
  mass = 10.0f;

  gBall->SetInertiaTensor(neSphereInertiaTensor(0.8f, mass));

  gBall->SetMass(mass);

	gBall->SetAngularDamping(0.01f);

		// set the starting position of ball
  pos.Set(0.0, -4.6, -10.0);
  gBall->SetPos(pos);

	/* **** End creation of white ball **** */


  /* **** Create rigid body for the 15 billiard balls **** */
	int x = 0;
	for(x=0;x<15;x++)
	{
		gBalls[x] = gSim->CreateRigidBody();

		// Add geometry to the body and set it to be a sphere with a diameter
		geom = gBalls[x]->AddGeometry();

		geom->SetSphereDiameter(0.8f);

		geom->SetMaterialIndex(1); // ball 
		// Update the bounding info of the object -- must always call this
		// after changing a body's geometry.
		gBalls[x]->UpdateBoundingInfo();

		// Set other properties of the object (mass, position, etc.)
		mass = 10.0f;

		gBalls[x]->SetInertiaTensor(neSphereInertiaTensor(0.8f, mass));

		gBalls[x]->SetMass(mass);

		gBalls[x]->SetAngularDamping(0.01f);

			// set the starting position of balls 
		switch(x)
		{
		//row 1
		case 0: pos.Set(0.0, -4.6, -20.0); break;
		//row 2
		case 1: pos.Set(-0.42, -4.6, -20.82); break;
		case 2: pos.Set(0.42, -4.6, -20.82); break;
		//row 3
		case 3: pos.Set(-0.82, -4.6, -21.62); break;
		case 4: pos.Set(0.0, -4.6, -21.62); break;
		case 5: pos.Set(0.82, -4.6, -21.62); break;
		//row 4
		case 6: pos.Set(-1.22, -4.6, -22.42); break;
		case 7: pos.Set(-0.42, -4.6, -22.42); break;
		case 8: pos.Set(0.42, -4.6, -22.42); break;
		case 9: pos.Set(1.22, -4.6, -22.42); break;
		//row 5
		case 10: pos.Set(-1.62, -4.6, -23.22); break;
		case 11: pos.Set(-0.82, -4.6, -23.22); break;
		case 12: pos.Set(0.0, -4.6, -23.22); break;
		case 13: pos.Set(0.82, -4.6, -23.22); break;
		case 14: pos.Set(1.62, -4.6, -23.22); break;
		}

		gBalls[x]->SetPos(pos);
  }
	/* **** End creation of the 15 billiard balls **** */

		
	// Create an animated body for the floor and walls
  gFloor = gSim->CreateAnimatedBody();
	gRwall = gSim->CreateAnimatedBody();
	gLwall = gSim->CreateAnimatedBody();
	gBwall = gSim->CreateAnimatedBody();
	gFwall = gSim->CreateAnimatedBody();

  /* **** 
	Add geometry to the floor and walls and set each to be a box with size as defined by the FLOORSIZE constant
  **** */  
		//floor
		geom = gFloor->AddGeometry();
    boxSize.Set((FLOORSIZE/2), 1.0f, FLOORSIZE);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gFloor->UpdateBoundingInfo();
		// Set the position of the floor within the simulator
    pos.Set(0.0, -5.5, -20.0);
    gFloor->SetPos(pos);

		//right wall
		geom = gRwall->AddGeometry();
    boxSize.Set(2.0f, 2.0f, FLOORSIZE);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gRwall->UpdateBoundingInfo();
		// Set the position of the right wall within the simulator
    pos.Set(10.0, -4.0, -20.0);
    gRwall->SetPos(pos);

		//left wall
		geom = gLwall->AddGeometry();
    boxSize.Set(2.0f, 2.0f, FLOORSIZE);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gLwall->UpdateBoundingInfo();
		// Set the position of the left wall within the simulator
    pos.Set(-10.0, -4.0, -20.0);
    gLwall->SetPos(pos);

		//back wall
		geom = gBwall->AddGeometry();
    boxSize.Set(FLOORSIZE/2, 2.0f, 2.0f);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gBwall->UpdateBoundingInfo();
		// Set the position of the back wall within the simulator
    pos.Set(0.0, -4.0, -40.0);
    gBwall->SetPos(pos);

		//front wall
		geom = gFwall->AddGeometry();
    boxSize.Set(FLOORSIZE/2, 2.0f, 2.0f);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gFwall->UpdateBoundingInfo();
		// Set the position of the front wall within the simulator
    pos.Set(0.0, -4.0, 0.0);
    gFwall->SetPos(pos);


    // All done
    return true;
}


void KillPhysics(void)
{
	if (gSim)
	{
		// Destroy the simulator.
		// Note that this will release all related resources that we've allocated.
		neSimulator::DestroySimulator(gSim);
		gSim = NULL;
	}
}


void light()
{
	GLfloat mat_specular[] = {0.0, 1.0, 1.0, 1.0};

	GLfloat mat_shininess[] = {75.0};
	GLfloat light_positionC[] = {0.0, 5.0, 13.0, 0.0};
	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat red_light[] = {1.0, 0.0, 0.0, 1.0};
	GLfloat lmodel_ambient[] = {0.5, 0.5, 0.5, 1.0};

	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	
	//LIGHT0
	glLightfv(GL_LIGHT0, GL_POSITION, light_positionC);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}




void init()
{
	glClearColor(0.0,0.0,0.0,0.0);		
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	

	if(useLight)
		light();


	//quadric objects created 
	q = gluNewQuadric();
	int x;
	for(x=0;x<15;x++)
		qBalls[x] = gluNewQuadric();

	InitPhysics();  
	InitTimer();
}


void reshape(int w, int h)
{
	glViewport(0, 0,(GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (GLfloat)w/(GLfloat)h, 1.0, 100.0);
	glMatrixMode(GL_MODELVIEW);
}


int main(int argc, char **argv)
{
  //////////
  // INIT //
  //////////

  // initialize glut
  glutInit(&argc, argv);

  // specify the display mode to be RGB and double buffering 
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

  // define the size
  glutInitWindowSize(500,500);

  // the position where the window will appear
  glutInitWindowPosition(100,100);

	//fullscreen
	//glutFullScreen();

  win = glutCreateWindow("Implementing Tokamak: Billiard");

	init();

  //////////////
  // CALLBACK //
  //////////////

  glutDisplayFunc(disp);
	glutReshapeFunc(reshape);
  glutIdleFunc(idle);
  glutKeyboardFunc(keyb);
	glutKeyboardUpFunc(keyUp);
	glutMouseFunc(mouseb);

  ////////////
  // OPENGL //
  ////////////

  // enter the main loop
  glutMainLoop();

	KillPhysics();
  return 0;
}


void disp(void)
{	
	float fElapsed;
	static float fLastElapsed;
	int x;

	/* **** TIMER **** */

	// Find out how much time has elapsed since we were last called
	fElapsed = GetElapsedTime();

	// In order to keep passing a reasonably stable value into the simulator's Advance
	// method, we'll try to stop occasional variations in the elapsed time from occurring.
	// We'll do this by preventing the elapsed time from being more than 20% greater or
	// less than the previous elapsed time.
	if (fLastElapsed != 0)
	{
		if (fElapsed > fLastElapsed * 1.2f) fElapsed = fLastElapsed * 1.2f;
		if (fElapsed < fLastElapsed * 0.8f) fElapsed = fLastElapsed * 0.8f;
	}
	// To avoid particularly large spikes, we'll stop the elapsed time from exceeding
	// 1/45th of a second. Values larger than this can lead to an unstable simulation.
	if (fElapsed > 1.0f / 45.0f) fElapsed = 1.0f / 45.0f;
	// Store the elapsed time so that we can use it the next time around
	fLastElapsed = fElapsed;

	// Tell the Tokamak simulator to advance all its calculations by one frame.
	// Note that we pass the elapsed time here so that the simulation speed
	// is constant regardless of the refresh frequency, frame rate, etc.
	//IMPORTANT: seems this now tells the Advance() function the amount of time that
	//has passed in SECONDS(I get like 0,005 if i write fElapsed out), not in MILLISECONDS
	//like the Tokamak Documentation says (might be an old documentation). 
	//If I pass the time in ms it behaves strangely.
	gSim->Advance(fElapsed);
	
	/* **** END TIMER **** */

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glLoadIdentity();

	glRotatef(anglx,rotx,0.0, 0.0);
	glRotatef(angly,0.0,roty, 0.0);

	glTranslatef(globalx,globaly,globalz);

	drawTable(0);
	
	glColor3f(1.0,1.0,1.0);
	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_ball);
	drawBall(gBall, 0.4f, q, 1);

	
	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_balls);
	
	for(x=0;x<15;x++)
	{
		drawBall(gBalls[x], 0.4f, qBalls[x], x+2); 
	}

	glutSwapBuffers();
	glFlush();
}


// DRAWING FUNCTIONS 

void drawTable(int texIndex)
{
	neT3 t;
	
	//floor
	t = gFloor->GetTransform(); 
	float floormat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};

  glPushMatrix();

	glMultMatrixf (floormat);

	glColor3f(0.0,0.3,0.0);
	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_table);

	drawCube(FLOORSIZE/2, 1.0, FLOORSIZE);
	
	glPopMatrix();	
	
	//all the walls

		
	glColor3f(0.5,0.3, 0.1);
	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_walls);

	//right wall
	t = gRwall->GetTransform(); 
	float rwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();

	glMultMatrixf (rwallmat);


	drawCube(2.0, 2.0, FLOORSIZE+2);

	glPopMatrix();
	
	//left wall
	t = gLwall->GetTransform(); 
	float lwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();
	
	glMultMatrixf (lwallmat);

	drawCube(2.0, 2.0, FLOORSIZE+2);
	
	glPopMatrix();
	
	//back wall
	t = gBwall->GetTransform(); 
	float bwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();
	
	glMultMatrixf (bwallmat);

	drawCube((FLOORSIZE/2)+2, 2.0, 2.0);
	
	glPopMatrix();
	
	//front wall
	t = gFwall->GetTransform(); 
	float fwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();
	
	glMultMatrixf (fwallmat);

	drawCube((FLOORSIZE/2)+2, 2.0, 2.0);

	glPopMatrix();
	
}


void drawBall(neRigidBody* localBall, float radius, GLUquadricObj* localQ, int texIndex)
{
	neT3 t;

	glPushMatrix();

	t = localBall->GetTransform();
	float mat[16] = 
			{
				t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
				t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
				t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
				t.pos[0],    t.pos[1],    t.pos[2],    1.0f
			};
	glMultMatrixf (mat);

	gluQuadricDrawStyle(localQ, GLU_FILL);

	gluQuadricNormals(localQ, GLU_SMOOTH);

	gluSphere (localQ, radius, 20, 20);
	glPopMatrix();
}


void drawCube(GLfloat w, GLfloat he, GLfloat len)
{
	glBegin(GL_QUADS);
	{
		//left
		glNormal3f(-1.0, 0.0, 0.0);
		glVertex3f(-w/2, he/2, len/2);
		glVertex3f(-w/2, he/2, -len/2);
		glVertex3f(-w/2, -he/2, -len/2);
		glVertex3f(-w/2, -he/2, len/2);

		//right
		glNormal3f(1.0, 0.0, 0.0);
		glVertex3f(w/2, he/2, len/2);
		glVertex3f(w/2, he/2, -len/2);
		glVertex3f(w/2, -he/2, -len/2);
		glVertex3f(w/2, -he/2, len/2);

		//front
		glNormal3f(0.0, 0.0, 1.0);
		glVertex3f(-w/2, he/2, len/2);
		glVertex3f(w/2, he/2, len/2);
		glVertex3f(w/2, -he/2, len/2);
		glVertex3f(-w/2, -he/2, len/2);

		//back
		glNormal3f(0.0, 0.0, -1.0);
		glVertex3f(-w/2, he/2, -len/2);
		glVertex3f(w/2, he/2, -len/2);
		glVertex3f(w/2, -he/2, -len/2);
		glVertex3f(-w/2, -he/2, -len/2);

		//top
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(-w/2, he/2, -len/2);
		glVertex3f(w/2, he/2, -len/2);
		glVertex3f(w/2, he/2, len/2);
		glVertex3f(-w/2, he/2, len/2);

		//bottom
		glNormal3f(0.0, -1.0, 0.0);
		glVertex3f(-w/2, -he/2, -len/2);
		glVertex3f(w/2, -he/2, -len/2);
		glVertex3f(w/2, -he/2, len/2);
		glVertex3f(-w/2, -he/2, len/2);

	}
	glEnd();

}


//INPUT FUNCTIONS

void idle()
{
	if(rot_right)
	{
	  roty = 1.0f;
	  angly = angly + 0.1;
	}
	if(rot_left)
	{
	  roty = 1.0f;
	  angly = angly - 0.1;
	}
	if(rot_up)
	{
	  rotx = 1.0f;
	  anglx = anglx + 0.1;	
	}
	if(rot_down)
	{
	  rotx = 1.0f;
	  anglx = anglx - 0.1;
	}
	if(xup)
	{
		globalx += 0.1;
	}
	if(xdown)
	{
		globalx -= 0.1;
	}
	if(yup)
	{
		globaly += 0.1;
	}
	if(ydown)
	{
		globaly -= 0.1;
	}
	if(zup)
	{
		globalz += 0.1;
	}
	if(zdown)
	{
		globalz -= 0.1;
	}
	if(imp[0] == 1)
		impz -= 1.0;
	if(imp[1] == 1)
		impz += 1.0;
	if(imp[2] == 1)
		impx -= 1.0;
	if(imp[3] == 1)
		impx += 1.0;

	//write rotation and zoom coords to screen
	//cout << anglx << ", " << angly << ", " << globalx << ", " << globaly << ", " << globalz << endl;
	glutPostRedisplay();
}


void keyUp(unsigned char key, int x, int y)
{
	if(key == '4')
		rot_right = false;
	if(key == '3')
		rot_left = false;
	if(key == '2')
		rot_up = false;
	if(key == '1')
		rot_down = false;
	if(key == '5')
		xup = false;
	if(key == '6')
		xdown = false;
	if(key == '7')
		yup = false;
	if(key == '8')
		ydown = false;
	if(key == '9')
		zup = false;
	if(key == '0')
		zdown = false;
	if(key == 'w' || key == 's' || key == 'a' || key == 'd')
	{
		imp[0] = 0;
		imp[1] = 0;
		imp[2] = 0;
		imp[3] = 0;
		impulse.Set(impx, impy, impz);
		gBall->ApplyImpulse(impulse);

		impx = impy =	impz = 0.0;
	}
}


void keyb(unsigned char key, int x, int y)
{
	
	
	if(key == 'q' || key == 'Q')
	{
		glutDestroyWindow(win);
		exit(0);
	}
	else if(key =='I' || key == 'i')
	{
		InitPhysics();
	}
	else if(key == 'd' || key == 'D')
	{
		imp[3] = 1;
	}
	else if(key == 'a' || key == 'A')
	{
		imp[2] = 1;
	}
	else if(key == 'w' || key == 'W')
	{
		imp[0] = 1;
	}
	else if(key == 's' || key == 'S')
	{
		imp[1] = 1;
	}
	else if(key == '5')
	{
		xup = true;
	}
	else if(key == '6')
	{
		xdown = true;
	}
	else if(key == '7')
	{
		yup = true;
	}
	else if(key == '8')
	{
		ydown = true;
	}
	else if(key == '9')
	{
		zup = true;
	}
	else if(key == '0')
	{
		zdown = true;
	}
	else if(key == '4')
	{
		rot_right = true;
	}
	else if(key == '3')
	{
		rot_left = true;
	}
	else if(key == '2')
	{
		rot_up = true;
	}
	else if(key == '1')
	{
		rot_down = true;
	}
}


void mouseb(int button, int state, int x, int y)
{
}