/***************************************************************
	LABYRINTH

	Filip Hammerstad. Spring 2004

	components: 
		-OpenGL
		-GLU
		-GLUT
		-Tokamak Physics library(www.tokamakphysics.com)
		-Kvil image texturizer(http://www.ia.hiof.no/~borres/gb/exp-textur/p-vcc-textur.html, courtesy of Mats Lindh)

*****************************************************************/

#include <iostream>
#include <cstdlib>
#include <GL/glut.h>
#include "../kvil/kvil.h"
#include "../tokamak/include/tokamak.h"
#include <time.h>

#define FLOORSIZE 20.0f

using namespace std;

bool useLight = true; //light or no light?
bool useTextures = false; //texture or no texture?

//materials
GLfloat mat_shininess[] = {50.0};
GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
GLfloat mat_table[] = {0.7, 0.4, 0.2, 1.0};
GLfloat mat_walls[] = {0.5, 0.3, 0.1, 1.0};
GLfloat mat_ball[] = {1.0, 1.0, 0.0, 1.0};
GLfloat mat_ball2[] = {1.0, 0.0, 0.0, 1.0};

//array for floorshifting:
GLfloat floorShift[9] = 
	{ 0.0, 0.0, 0.0,
		0.0, 0.0, 0.0,
		0.0, 0.0, 0.0};

//values for twisting the floor
GLfloat twistx = 0.0f;
GLfloat twisty = 0.0f;
GLfloat twistz = 0.0f;

neV3 twist = {twistx, twisty, twistz};

// Timer variables
bool gbUseHFTimer;
INT64 gCurrentTime;
float gfTimeScale;

//the simulator
neSimulator* gSim = NULL;

neV3 gravity = {0.0,0.0,0.0};	// A vector to store the direction and intensity of gravity

//the physics bodies
neRigidBody* gBall = NULL;
neRigidBody* gBall2 = NULL;
neAnimatedBody* gFloor = NULL;
neAnimatedBody* gRwall = NULL;
neAnimatedBody* gLwall = NULL;
neAnimatedBody* gBwall = NULL;
neAnimatedBody* gFwall = NULL;

static int win;

//translate and rotate variables

GLfloat globalx = -7.0;
GLfloat globaly = -25.0;
GLfloat globalz = -25.0;
GLfloat rotx = 1.0;
GLfloat roty = 1.0;
GLfloat anglx = 32.9;
GLfloat angly = -11.2;

//bool variables for which rotations and translations are active, used in idle and keyboard function
bool rot_up,rot_down,rot_right,rot_left,xup,xdown,yup,ydown,zup,zdown = false;
 
//quadric object (sphere)
GLUquadricObj* q;
GLUquadricObj* q2;

GLuint textures[4]; //texture array

//kvil objects
kvil* kv_wall;
kvil* kv_sideWalls;
kvil* kv_ball;
kvil* kv_ball2;

// function prototypes
void disp(void);
void idle(void);
void keyb(unsigned char key, int x, int y);
void keyUp(unsigned char key, int x, int y);
void mouseb(int button, int state, int x, int y);
void reshape(int w, int h);
void drawTable(int texIndex); //draws alle the walls and the floor of the room
void drawBall(neRigidBody* localBall, float radius, GLUquadricObj* localQ, int texIndex); // draws a ball
void drawCube(GLfloat w, GLfloat he, GLfloat len); //draws a cube of w width(x), he height(y) and len length(z)


// Initialise a timer for use in our application.
// Wherever possible use a Performance timer as this
// gives the best accuracy.
// If this isn't available, fall back to the multimedia timer.
// In each case, note the number of "ticks" per second so that
// we can divide the elapsed time by this value to determine
// how much real time has actually passed.
// For more information on using these timers, see the articles at
// www.mvps.org/directx
bool InitTimer(void)
{
	INT64 TimerFrequency;

	// Can we use the high performance timer?
	if (QueryPerformanceFrequency( (LARGE_INTEGER*)&TimerFrequency))
	{
		// High performance timer available, let's get ready to use it
		gfTimeScale = 1.0f / TimerFrequency;
		QueryPerformanceCounter((LARGE_INTEGER *) &gCurrentTime);
		gbUseHFTimer = true;
		return true;
	}
	else
	{
		// No high precision timer to be seen, let's use the Windows multimedia timer instead
		gfTimeScale = 0.001f;
		gCurrentTime = timeGetTime();
		gbUseHFTimer = false;
		return true;
	}

	// No timer available at all (should never happen)
	return false;
}


// Use the timer we previously initialised to determine
// how much time (in seconds, or a fraction thereof) has
// passed since this function was last called.
float GetElapsedTime()
{
	INT64 newTime;
	float fElapsed;

	if (gbUseHFTimer) 
	{
		QueryPerformanceCounter((LARGE_INTEGER *) &newTime);
	}
	else 
	{
		newTime=timeGetTime();
	}

	// Scale accordingly
	fElapsed = (float)((newTime - gCurrentTime) * gfTimeScale);

	// Save the new time value for the next time we're called
	gCurrentTime = newTime;

	return fElapsed;
}


// Initialise the Tokamak physics engine.
bool InitPhysics(void)
{
  neGeometry *geom;	// A Geometry object which used to define the shape/size of objects
  neV3 boxSize;		// A variable to store the length, width and height of the boxes (floors and walls)
  
  neV3 pos;			// The position
  f32 mass;			// The mass 
  neSimulatorSizeInfo sizeInfo;	// Stores data about how many objects we are going to model


  // Create and initialise the simulator
  // Tell the simulator how many rigid bodies we have
  sizeInfo.rigidBodiesCount = 2;
  // Tell the simulator how many animated bodies we have
  sizeInfo.animatedBodiesCount = 6;
  // Tell the simulator how many bodies we have in total
  s32 totalBody = sizeInfo.rigidBodiesCount + sizeInfo.animatedBodiesCount;
  sizeInfo.geometriesCount = totalBody;
  // The overlapped pairs count defines how many bodies it is possible to be in collision
  // at a single time. The SDK states this should be calculated as:
  //   bodies * (bodies-1) / 2
  // So we'll take its word for it. :-)
  sizeInfo.overlappedPairsCount = totalBody * (totalBody - 1) / 2;
  
	// We're not using any of these so set them all to zero
  sizeInfo.rigidParticleCount = 0;
  sizeInfo.constraintsCount = 0;
  sizeInfo.terrainNodesStartCount = 0;

	// Set the gravity. Try changing this to see the effect on the objects
  gravity.Set(0.0f, -10.0f, 0.0f);
	

	// Ready to go, create the simulator object
  gSim = neSimulator::CreateSimulator(sizeInfo, NULL, &gravity);
	
	//set materials in the simulator object
	gSim->SetMaterial(0, 0.5, 0.5); //table material 
	gSim->SetMaterial(1, 0.5, 0.5); //ball material


	/* **** Create rigid body for ball 1 **** */

  gBall = gSim->CreateRigidBody();

	// Add geometry to the body and set it to be a sphere with a diameter
  geom = gBall->AddGeometry();

  geom->SetSphereDiameter(0.8f);

	geom->SetMaterialIndex(1); //ball material
  // Update the bounding info of the object -- must always call this
  // after changing a body's geometry.
  gBall->UpdateBoundingInfo();

	// Set other properties of the object (mass, position, etc.)
  mass = 10.0f;

  gBall->SetInertiaTensor(neSphereInertiaTensor(0.8f, mass));

  gBall->SetMass(mass);

	gBall->SetAngularDamping(0.01f);

		// set the starting position of ball 1
  pos.Set(0.0, -5.6, -15.0);
  gBall->SetPos(pos);

	/* **** End creation of ball 1 **** */


  /* **** Create rigid body for ball 2 **** */
  gBall2 = gSim->CreateRigidBody();

	// Add geometry to the body and set it to be a sphere with a diameter
  geom = gBall2->AddGeometry();

  geom->SetSphereDiameter(0.8f);

	geom->SetMaterialIndex(1); // ball material
  // Update the bounding info of the object -- must always call this
  // after changing a body's geometry.
  gBall2->UpdateBoundingInfo();

	// Set other properties of the object (mass, position, etc.)
  mass = 10.0f;

  gBall2->SetInertiaTensor(neSphereInertiaTensor(0.8f, mass));

  gBall2->SetMass(mass);

	gBall2->SetAngularDamping(0.01f);

		// set the starting position of ball 2
  pos.Set(0.00, -5.6, -10.0);
  gBall2->SetPos(pos);
  
	/* **** End creation of ball 2 **** */
		
	// Create the animated bodyies 
  gFloor = gSim->CreateAnimatedBody();
	gRwall = gSim->CreateAnimatedBody();
	gLwall = gSim->CreateAnimatedBody();
	gBwall = gSim->CreateAnimatedBody();
	gFwall = gSim->CreateAnimatedBody();




		//floor
		geom = gFloor->AddGeometry();
    boxSize.Set(FLOORSIZE, 2.0f, FLOORSIZE);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gFloor->UpdateBoundingInfo();

		// Set the position of the floor within the simulator
    pos.Set(0.0, -7.0, -FLOORSIZE/2);
    gFloor->SetPos(pos);

		//right wall
		geom = gRwall->AddGeometry();
    boxSize.Set(1.0f, 6.0f, FLOORSIZE);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gRwall->UpdateBoundingInfo();
		// Set the position of the right wall within the simulator
    pos.Set((FLOORSIZE/2), -6.0, -FLOORSIZE/2);
    gRwall->SetPos(pos);

		//left wall
		geom = gLwall->AddGeometry();
    boxSize.Set(1.0f, 6.0f, FLOORSIZE);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gLwall->UpdateBoundingInfo();
		// Set the position of the left wall within the simulator
    pos.Set(-FLOORSIZE/2, -6.0, -FLOORSIZE/2);
    gLwall->SetPos(pos);

		//back wall
		geom = gBwall->AddGeometry();
    boxSize.Set(FLOORSIZE, 6.0f, 1.0f);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gBwall->UpdateBoundingInfo();
		// Set the position of the back wall within the simulator
    pos.Set(0.0, -6.0, -FLOORSIZE);
    gBwall->SetPos(pos);

		//front wall
		geom = gFwall->AddGeometry();
    boxSize.Set(FLOORSIZE, 6.0f, 1.0f);
    geom->SetBoxSize(boxSize[0],boxSize[1],boxSize[2]);
		geom->SetMaterialIndex(0); //brick material
    gFwall->UpdateBoundingInfo();
		// Set the position of the front wall within the simulator
    pos.Set(0.0, -6.0, 0.0);
    gFwall->SetPos(pos);

    // All done
    return true;
}


void KillPhysics(void)
{
	if (gSim)
	{
		// Destroy the simulator.
		// Note that this will release all related resources that we've allocated.
		neSimulator::DestroySimulator(gSim);
		gSim = NULL;
	}
}


void texture()
{
	kv_wall = new kvil();
	kv_sideWalls = new kvil();
	glGenTextures(1, &textures[0]);
	glGenTextures(1, &textures[1]);
	kv_wall->LoadGL(textures[0], "../images/wood.bmp");
	kv_sideWalls->LoadGL(textures[1], "../images/wood2.bmp");

	kv_ball = new kvil();
	kv_ball2 = new kvil();

	glGenTextures(1, &textures[2]);
	glGenTextures(1, &textures[3]);

	kv_ball->LoadGL(textures[2], "../images/met.bmp");
	kv_ball2->LoadGL(textures[3], "../images/met2.bmp");
	
	glEnable(GL_TEXTURE_2D);	
}


void light()
{
	GLfloat light_positionC[] = {0.0, 5.0, 13.0, 0.0};
	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat red_light[] = {1.0, 0.0, 0.0, 1.0};
	GLfloat lmodel_ambient[] = {0.5, 0.5, 0.5, 0.0};

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	
	//LIGHT0
	glLightfv(GL_LIGHT0, GL_POSITION, light_positionC);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}


void init()
{
	glClearColor(0.0,0.0,0.0,0.0);		
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	
	if(useTextures)
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	light();

	if(useTextures)
		texture();

	//quadric object (sphere)
	q = gluNewQuadric();
	q2 = gluNewQuadric();

	InitPhysics();  
	InitTimer();
}


void reshape(int w, int h)
{
	glViewport(0, 0,(GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (GLfloat)w/(GLfloat)h, 1.0, 100.0);
	glMatrixMode(GL_MODELVIEW);
}


int main(int argc, char **argv)
{
  //////////
  // INIT //
  //////////

  // initialize glut
  glutInit(&argc, argv);

  // specify the display mode to be RGB and double buffering 
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

  // define the size
  glutInitWindowSize(500,500);

  // the position where the window will appear
  glutInitWindowPosition(100,100);

  win = glutCreateWindow("Implementing Tokamak: Labyrinth");

	init();

  //////////////
  // CALLBACK //
  //////////////

  glutDisplayFunc(disp);
	glutReshapeFunc(reshape);
  glutIdleFunc(idle);
  glutKeyboardFunc(keyb);
	glutKeyboardUpFunc(keyUp);
	glutMouseFunc(mouseb);

  ////////////
  // OPENGL //
  ////////////

  // enter the main loop
  glutMainLoop();

	KillPhysics();
  return 0;
}


void disp(void)
{	
	float fElapsed;
	static float fLastElapsed;

	/* **** TIMER**** */

	// Find out how much time has elapsed since we were last called
	fElapsed = GetElapsedTime();

	// In order to keep passing a reasonably stable value into the simulator's Advance
	// method, we'll try to stop occasional variations in the elapsed time from occurring.
	// We'll do this by preventing the elapsed time from being more than 20% greater or
	// less than the previous elapsed time.
	if (fLastElapsed != 0)
	{
		if (fElapsed > fLastElapsed * 1.2f) fElapsed = fLastElapsed * 1.2f;
		if (fElapsed < fLastElapsed * 0.8f) fElapsed = fLastElapsed * 0.8f;
	}
	// To avoid particularly large spikes, we'll stop the elapsed time from exceeding
	// 1/45th of a second. Values larger than this can lead to an unstable simulation.
	if (fElapsed > 1.0f / 45.0f) fElapsed = 1.0f / 45.0f;
	// Store the elapsed time so that we can use it the next time around
	fLastElapsed = fElapsed;

	// Tell the Tokamak simulator to advance all its calculations by one frame.
	// Note that we pass the elapsed time here so that the simulation speed
	// is constant regardless of the refresh frequency, frame rate, etc.
	//IMPORTANT: seems this now tells the Advance() function the amount of time that
	//has passed in SECONDS(I get like 0,005 if i write fElapsed out), not in MILLISECONDS
	//like the Tokamak Documentation says (might be an old documentation). 
	//If I pass the time in ms it behaves strangely.
	gSim->Advance(fElapsed);
	
	/* **** END TIMER **** */

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glLoadIdentity();

	glRotatef(anglx,rotx,0.0, 0.0);
	glRotatef(angly,0.0,roty, 0.0);

	glTranslatef(globalx,globaly,globalz);

	drawTable(0);

	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_ball);
	drawBall(gBall, 0.4, q, 2);
	
	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_ball2);
	drawBall(gBall2, 0.4, q2, 3); 

	glutSwapBuffers();
	glFlush();
}


// DRAWING FUNCTIONS 

void drawTable(int texIndex)
{
	neT3 t;
	neQ rot;

	if(useTextures)
		glBindTexture(GL_TEXTURE_2D, textures[texIndex]);

	//floor
	t = gFloor->GetTransform(); 
	float floormat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	rot = gFloor->GetRotationQ();
	
	rot.X += twistx;
	rot.Y += twisty;
	rot.Z += twistz;

	gFloor->SetRotation(rot);

  glPushMatrix();

	glMultMatrixf (floormat);

	//color for all the walls and floor
	glColor3f(0.5,0.3, 0.1);

	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_table);

	drawCube(FLOORSIZE, 2.0, FLOORSIZE);

	glPopMatrix();	


	//all the walls uses texture from the texture array index of the floor +1
	//make sure to have a texture loaded at both the floors array index and the one after!!
	if(useTextures)
		glBindTexture(GL_TEXTURE_2D, textures[texIndex+1]);

	//right wall
	t = gRwall->GetTransform(); 
	float rwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();

	glMultMatrixf (rwallmat);

	if(useLight)
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_walls);

	drawCube(1.0, 6.0, FLOORSIZE+1);

	glPopMatrix();
	
	//left wall
	t = gLwall->GetTransform(); 
	float lwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();
	
	glMultMatrixf (lwallmat);

	drawCube(1.0, 6.0, FLOORSIZE+1);
	
	glPopMatrix();
	
	//back wall
	t = gBwall->GetTransform(); 
	float bwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();
	
	glMultMatrixf (bwallmat);

	drawCube(FLOORSIZE+1, 6.0, 1.0);
	
	glPopMatrix();
	
	//front wall
	t = gFwall->GetTransform(); 
	float fwallmat[16] = 
	{
		t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
		t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
		t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
		t.pos[0],    t.pos[1],    t.pos[2],    1.0f
	};
	
	glPushMatrix();
	
	glMultMatrixf (fwallmat);

	drawCube(FLOORSIZE+1, 6.0, 1.0);

	glPopMatrix();
	
}


void drawBall(neRigidBody* localBall, float radius, GLUquadricObj* localQ, int texIndex)
{
	neT3 t;

	glPushMatrix();

	t = localBall->GetTransform();
	float mat[16] = 
			{
				t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
				t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
				t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
				t.pos[0],    t.pos[1],    t.pos[2],    1.0f
			};
	glMultMatrixf (mat);

	gluQuadricDrawStyle(localQ, GLU_FILL);

	gluQuadricNormals(localQ, GLU_SMOOTH);
	
	if(useTextures)
	{
		glBindTexture(GL_TEXTURE_2D, textures[texIndex]);
		gluQuadricTexture(localQ, GL_TRUE);
	}

	glColor3f(0.0,0.0,0.0);

	gluSphere (q, radius, 20, 20);
	glPopMatrix();
}


void drawCube(GLfloat w, GLfloat he, GLfloat len)
{
	glBegin(GL_QUADS);
	{
		//left
		glNormal3f(-1.0, 0.0, 0.0);
			if(useTextures)
				glTexCoord2f(0.0f, 1.0f);	
		glVertex3f(-w/2, he/2, len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 1.0f);	
		glVertex3f(-w/2, he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 0.0f);	
		glVertex3f(-w/2, -he/2, -len/2);
			if(useTextures)
				glTexCoord2f(0.0f, 0.0f);	
		glVertex3f(-w/2, -he/2, len/2);

		//right
		glNormal3f(1.0, 0.0, 0.0);
			if(useTextures)
				glTexCoord2f(0.0f, 1.0f);	
		glVertex3f(w/2, he/2, len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 1.0f);	
		glVertex3f(w/2, he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 0.0f);	
		glVertex3f(w/2, -he/2, -len/2);
			if(useTextures)
				glTexCoord2f(0.0f, 0.0f);	
		glVertex3f(w/2, -he/2, len/2);

		//front
		glNormal3f(0.0, 0.0, 1.0);
			if(useTextures)
				glTexCoord2f(0.0f, 1.0f);	
		glVertex3f(-w/2, he/2, len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 1.0f);	
		glVertex3f(w/2, he/2, len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 0.0f);	
		glVertex3f(w/2, -he/2, len/2);
			if(useTextures)
				glTexCoord2f(0.0f, 0.0f);	
		glVertex3f(-w/2, -he/2, len/2);

		//back
		glNormal3f(0.0, 0.0, -1.0);
			if(useTextures)
				glTexCoord2f(0.0f, 1.0f);	
		glVertex3f(-w/2, he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 1.0f);	
		glVertex3f(w/2, he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 0.0f);	
		glVertex3f(w/2, -he/2, -len/2);
			if(useTextures)
				glTexCoord2f(0.0f, 0.0f);	
		glVertex3f(-w/2, -he/2, -len/2);

		//top
		glNormal3f(0.0, 1.0, 0.0);
			if(useTextures)
				glTexCoord2f(0.0f, 1.0f);	
		glVertex3f(-w/2, he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 1.0f);	
		glVertex3f(w/2, he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 0.0f);	
		glVertex3f(w/2, he/2, len/2);
			if(useTextures)
				glTexCoord2f(0.0f, 0.0f);	
		glVertex3f(-w/2, he/2, len/2);

		//bottom
		glNormal3f(0.0, -1.0, 0.0);
			if(useTextures)
				glTexCoord2f(0.0f, 1.0f);	
		glVertex3f(-w/2, -he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 1.0f);	
		glVertex3f(w/2, -he/2, -len/2);
			if(useTextures)
				glTexCoord2f(1.0f, 0.0f);	
		glVertex3f(w/2, -he/2, len/2);
			if(useTextures)
				glTexCoord2f(0.0f, 0.0f);	
		glVertex3f(-w/2, -he/2, len/2);

	}
	glEnd();
}


//INPUT FUNCTIONS

void idle()
{
	if(rot_right)
	{
	  roty = 1.0f;
	  angly = angly + 0.1;
	}
	if(rot_left)
	{
	  roty = 1.0f;
	  angly = angly - 0.1;
	}
	if(rot_up)
	{
	  rotx = 1.0f;
	  anglx = anglx + 0.1;	
	}
	if(rot_down)
	{
	  rotx = 1.0f;
	  anglx = anglx - 0.1;
	}
	if(xup)
	{
		globalx += 0.1;
	}
	if(xdown)
	{
		globalx -= 0.1;
	}
	if(yup)
	{
		globaly += 0.1;
	}
	if(ydown)
	{
		globaly -= 0.1;
	}
	if(zup)
	{
		globalz += 0.1;
	}
	if(zdown)
	{
		globalz -= 0.1;
	}
	if(floorShift[3] == 1.0)
	{
		twistz = 0.0005f;
	}
	if(floorShift[5] == 1.0)
	{
		twistz = -0.0005f;
	}
	if(floorShift[1] == 1.0)
	{
		twistx = -0.0005f;
	}
	if(floorShift[4] == 1.0)
	{
		twistx = 0.0005f;
	}
	//write rotation and zoom coords to screen
	//cout << anglx << ", " << angly << ", " << globalx << ", " << globaly << ", " << globalz << endl;

	glutPostRedisplay();
}


void keyUp(unsigned char key, int x, int y)
{
	if(key == '4')
		rot_right = false;
	if(key == '3')
		rot_left = false;
	if(key == '2')
		rot_up = false;
	if(key == '1')
		rot_down = false;
	if(key == '5')
		xup = false;
	if(key == '6')
		xdown = false;
	if(key == '7')
		yup = false;
	if(key == '8')
		ydown = false;
	if(key == '9')
		zup = false;
	if(key == '0')
		zdown = false;
	if(key == 'a' || key == 'A')
	{
		floorShift[3] = 0.0;
		twistz = 0.0f;
	}
	if(key == 'd' || key == 'D')
	{
		floorShift[5] = 0.0;
		twistz = 0.0f;
	}
	if(key == 'w' || key == 'W')
	{
		floorShift[1] = 0.0;
		twistx = 0.0f;
	}
	if(key == 's' || key == 'S')
	{
		floorShift[4] = 0.0;
		twistx = 0.0f;
	}
			
	
}


void keyb(unsigned char key, int x, int y)
{
	if(key == 'q' || key == 'Q')
	{
		glutDestroyWindow(win);
		exit(0);
	}
	else if(key =='I' || key == 'i')
	{
		InitPhysics();
	}
	else if(key == 'd' || key == 'D')
	{
		floorShift[5] = 1.0;
	}
	else if(key == 'a' || key == 'A')
	{
		floorShift[3] = 1.0;
	}
	else if(key == 'w' || key == 'W')
	{
		floorShift[1] = 1.0;
	}
	else if(key == 's' || key == 'S')
	{
		floorShift[4] = 1.0;
	}
	else if(key == '5')
	{
		xup = true;
	}
	else if(key == '6')
	{
		xdown = true;
	}
	else if(key == '7')
	{
		yup = true;
	}
	else if(key == '8')
	{
		ydown = true;
	}
	else if(key == '9')
	{
		zup = true;
	}
	else if(key == '0')
	{
		zdown = true;
	}
	else if(key == '4')
	{
		rot_right = true;
	}
	else if(key == '3')
	{
		rot_left = true;
	}
	else if(key == '2')
	{
		rot_up = true;
	}
	else if(key == '1')
	{
		rot_down = true;
	}
}


void mouseb(int button, int state, int x, int y)
{
}
